#include "globals.h"


//---------------------------------------------------------------------------------------------------


void sigio_handler( int fd, void *data ){
	instr_stop(1);
}

	
void sigchld_handler( int x ){
//	shutdown_bochs();
quit();
}


void quit( ){
	char temp_string[200];

	if(history_repeat){
		sprintf(temp_string, "\t\t\t\t\t*** repeated %ld times", history_repeat);
		if(history_logfile != NULL) fprintf(history_logfile, "%s\n", temp_string);
	}
	if(history_logfile != NULL) fclose(history_logfile);
	shutdown_bochs();
	printf("\n");
	exit(EXIT_SUCCESS);
}


void shutdown_bochs( ){
	signal(SIGCHLD, SIG_IGN);

	if(bochs_running){
		fl_remove_io_callback(fileno(readpipe), FL_READ, sigio_handler);
		if(!bochs_offline) kill(pid, SIGINT);
	}

	if(!bochs_offline) fprintf(writepipe, "q\n");

	fclose(readpipe);
	fclose(writepipe);

	fl_set_input(i_instruction_text, "  .. NOT CONNECTED TO BOCHS ..");
	fl_set_input_color(i_instruction_text, FL_RED, FL_RED);
	fl_addto_browser(i_list, "\t\t\t\t\tBOCHS OFFLINE...");
	fl_deactivate_object(go_button);
	fl_deactivate_object(stop_button);
	fl_deactivate_object(step_button);
	fl_deactivate_object(next_button);
	fl_deactivate_object(refreshall_button);
//	fl_deactivate_object(lock_button);
	fl_deactivate_object(folder);

//	bochs_offline = 1;
}


void start_bochs( ){
	signal(SIGCHLD, sigchld_handler);
	if(!popen2(BOCHS_PATH, &writepipe, &readpipe)){
		printf("PANIC: popen2() returned -1\n");
		exit(EXIT_FAILURE);
	}
	setbuf(readpipe, NULL);		// disable line buffering
	setbuf(writepipe, NULL);
//	sleep(1);
	bochs_offline = 0;
}


//---------------------------------------------------------------------------------------------------

int main( int argc, char *argv[] ){
	char temp_string[200];
	int x;
	struct memwin_struct *m;

	next_active = 0;
	bochs_running = 0;
	update_state = 1;
	history_repeat = 0;
	strcpy(history_lastline, "");

	if((history_logfile=fopen(HISTORY_PATH, "w")) == NULL){
		printf("Unable to access history log file, logging disabled.\n");
	}
	fl_initialize(&argc, argv, "Bochs Frontend", 0, 0);



#include "forms.inc"



	//-----------------------------
	// main form
	//-----------------------------
	window = fl_bgn_form(FL_FLAT_BOX, 580, 535);
	//-----------------------------
	// instruction widgets
	//-----------------------------
main_group = fl_bgn_group();
	i_address_text = fl_add_input(FL_NORMAL_INPUT,       5, 5,  75, 25, NULL);
	i_offset_text = fl_add_input(FL_NORMAL_INPUT,       85, 5, 115, 25, NULL);
	i_instruction_text = fl_add_input(FL_NORMAL_INPUT, 205, 5, 205, 25, NULL);
	fl_deactivate_object(i_address_text);
	fl_deactivate_object(i_offset_text);
	fl_deactivate_object(i_instruction_text);
	fl_set_object_lstyle(i_address_text, FL_BOLD_STYLE);
	fl_set_object_lstyle(i_offset_text, FL_BOLD_STYLE);
	fl_set_object_lstyle(i_instruction_text, FL_BOLD_STYLE);

	fl_set_object_gravity(i_address_text, FL_North, FL_North);
	fl_set_object_gravity(i_offset_text, FL_North, FL_North);
	fl_set_object_gravity(i_instruction_text, FL_North, FL_North);
	fl_set_object_resize(i_address_text, FL_RESIZE_NONE);
	fl_set_object_resize(i_offset_text, FL_RESIZE_NONE);
	fl_set_object_resize(i_instruction_text, FL_RESIZE_NONE);


	go_button = fl_add_button(FL_NORMAL_BUTTON,    415, 5,  40, 25, "Go");
	stop_button = fl_add_button(FL_NORMAL_BUTTON,  455, 5,  40, 25, "Stop");
	step_button = fl_add_button(FL_NORMAL_BUTTON,  495, 5,  40, 25, "Step");
	next_button = fl_add_button(FL_NORMAL_BUTTON,  535, 5,  40, 25, "Next");
	fl_set_object_lstyle(go_button, FL_BOLD_STYLE);
	fl_set_object_lstyle(stop_button, FL_BOLD_STYLE);
	fl_set_object_lstyle(step_button, FL_BOLD_STYLE);
	fl_set_object_lstyle(next_button, FL_BOLD_STYLE);
	fl_set_object_gravity(go_button, FL_North, FL_North);
	fl_set_object_gravity(stop_button, FL_North, FL_North);
	fl_set_object_gravity(step_button, FL_North, FL_North);
	fl_set_object_gravity(next_button, FL_North, FL_North);
	fl_set_object_resize(go_button, FL_RESIZE_NONE);
	fl_set_object_resize(stop_button, FL_RESIZE_NONE);
	fl_set_object_resize(step_button, FL_RESIZE_NONE);
	fl_set_object_resize(next_button, FL_RESIZE_NONE);
	fl_deactivate_object(stop_button);
	fl_set_object_lcol(stop_button, FL_BOTTOM_BCOL);



	//-----------------------------
	// misc widgets
	//-----------------------------
	folder = fl_add_tabfolder(FL_BOTTOM_TABFOLDER,         5,  35, 520, 158, NULL);
	fl_addto_tabfolder(folder, "Breakpoints", f_breakpoints_folder);
	fl_addto_tabfolder(folder, "Memory", f_memory_folder);
	fl_addto_tabfolder(folder, "Structures", f_structures_folder);
	fl_addto_tabfolder(folder, "Config", f_config_folder);
	fl_addto_tabfolder(folder, "History", f_history_folder);
	fl_set_object_gravity(folder, FL_North, FL_North);
	fl_set_object_resize(folder, FL_RESIZE_NONE);

	refreshall_button = fl_add_button(FL_NORMAL_BUTTON,      528,  90,  50, 35, "Refresh\nAll");
	fl_set_object_lstyle(refreshall_button, FL_BOLD_STYLE);
        fl_set_object_gravity(refreshall_button, FL_North, FL_North);
        fl_set_object_resize(refreshall_button, FL_RESIZE_NONE);

	dummy = fl_add_input(FL_NORMAL_INPUT,                      531, 135,  44, 29, NULL);
	quit_button = fl_add_button(FL_NORMAL_BUTTON,      533, 137,  40, 25, "Quit");
	fl_set_object_lstyle(quit_button, FL_BOLD_STYLE);
	fl_set_object_lcol(quit_button, FL_RED);
	fl_deactivate_object(dummy);
        fl_set_object_gravity(dummy, FL_North, FL_North);
        fl_set_object_gravity(quit_button, FL_North, FL_North);
        fl_set_object_resize(dummy, FL_RESIZE_NONE);
        fl_set_object_resize(quit_button, FL_RESIZE_NONE);

	hide_button = fl_add_button(FL_NORMAL_BUTTON,      533, 168,  40, 25, "Hide");
	show_button = fl_add_button(FL_NORMAL_BUTTON,      533, 168,  40, 25, "Show");
	fl_set_object_lstyle(hide_button, FL_BOLD_STYLE);
	fl_set_object_lstyle(show_button, FL_BOLD_STYLE);
	fl_hide_object(show_button);
        fl_set_object_gravity(hide_button, FL_North, FL_North);
        fl_set_object_gravity(show_button, FL_North, FL_North);
        fl_set_object_resize(hide_button, FL_RESIZE_NONE);
        fl_set_object_resize(show_button, FL_RESIZE_NONE);

fl_end_group();
fl_set_object_resize(main_group, FL_RESIZE_NONE);

#include "state.inc"



	//-----------------------------
	// end
	//-----------------------------
	fl_end_form();
	fl_show_form(window, FL_PLACE_MOUSE, FL_FULLBORDER, "Bochs Frontend");

#ifdef __DEBUG__
	printf("DEBUG: starting bochs... ");
#endif
	start_bochs();
#ifdef __DEBUG__
	printf("OK\n");
#endif
	prompt_read();

	// defaults settings
	history_length = 50;
	stack_above = 8;
	stack_below = 4;
	steplock_time = 2;  
	nextlock_time = 2;

        if((config=fopen(CONFIG_PATH, "r")) == NULL){
                printf("Unable to find config file, using defaults\n");
        } else {
		config_read(config);
	}

	// display initial config panel values
	sprintf(temp_string, "%d", history_length);
	fl_set_input(c_hislen_text, temp_string);
        sprintf(temp_string, "%d", stack_above);
        fl_set_input(c_sabove_text, temp_string);
        sprintf(temp_string, "%d", stack_below);
        fl_set_input(c_sbelow_text, temp_string);
        sprintf(temp_string, "%d", steplock_time);
        fl_set_input(c_steplock_text, temp_string);
        sprintf(temp_string, "%d", nextlock_time);
        fl_set_input(c_nextlock_text, temp_string);


	// generate structure listbox
	for(x=0; x < nr_strucs; x++){
		sprintf(temp_string, "@bbase=0x%.8X\telements=%d\tvalues=%d\tname=\"%s\"",
			memlist[x].base, memlist[x].element_count, memlist[x].value_count, memlist[x].name);
		fl_add_browser_line(st_list, temp_string);
		memlist[x].window = NULL;
	}
	mem_windows = NULL;

        if(!bochs_offline){
//		fprintf(writepipe, "load-symbols \"kernel.sym\" 0x5000\n");
//		prompt_read();
//		fprintf(writepipe, "load-symbols \"memory.sym\"\n");
//		prompt_read();
		fprintf(writepipe, "s\n");
	}
	update_xforms();


	do{
		event = fl_do_forms();

		if(event==go_button)		instr_go();
		if(event==step_button)		instr_step();
		if(event==next_button)		instr_next();
		if(event==stop_button)		instr_stop(0);

		if(event==physical_check)	bp_type = BREAKPOINT_PHYSICAL;
		if(event==linear_check)		bp_type = BREAKPOINT_LINEAR;
		if(event==virtual_check)	bp_type = BREAKPOINT_VIRTUAL;

		if(event==refreshall_button){
			if(bochs_running){
				fl_ringbell(0);
			} else {
			for(x=0; x < nr_strucs; x++) update_structure(x);
			m = mem_windows;
			while(m != NULL){
				update_memory((long)m);
				m = m->next;
			}}
		}
		if(event==quit_button)		ConfirmQuit();
		if(event==hide_button){
						fl_freeze_form(window);
						fl_hide_object(hide_button);
						fl_hide_object(state_widgets);
						fl_show_object(show_button);
						update_state = 0;
						fl_set_form_size(window, 580, 200);
						fl_unfreeze_form(window);
		}
		if(event==show_button){
						fl_freeze_form(window);
						fl_hide_object(show_button);
						fl_show_object(hide_button);
						fl_show_object(state_widgets);
						update_state = 1;
						fl_set_form_size(window, 580, 535);
						fl_unfreeze_form(window);
		}
	}while(1);
}
