#include "parse.h"


int breakpoints_read ( ){
	int x;
	int inputChar;
	char scanChar;

	if(bochs_offline) return(0);

	for(inputChar = fgetc(readpipe);
		(isdigit(inputChar) == 0) && (inputChar != '<') && (inputChar != EOF);
		inputChar = fgetc(readpipe)
		);
	if(inputChar == EOF) return(0);
	if(ungetc(inputChar, readpipe) == EOF) return(0);
	for(x = 0; x < 30; x++){
		for(inputChar = fgetc(readpipe);
			(isspace(inputChar) != 0) && (inputChar != EOF);
			inputChar = fgetc(readpipe)
			);
		if(inputChar == EOF) return(x);
		if((ungetc(inputChar, readpipe) == EOF) || (isdigit(inputChar) == 0)) return(x);
		if(fscanf(readpipe, "%i %c%*s ", &breakpoints[x].number, &scanChar) != 2) return(x);
		switch(scanChar){
			case 'l':
					breakpoints[x].type = BREAKPOINT_LINEAR;
					break;
			case 'p':
					breakpoints[x].type = BREAKPOINT_PHYSICAL;
					break;
			case 'v':
					breakpoints[x].type = BREAKPOINT_VIRTUAL;
					break;
			default:
					return(x);
		}
		if(fscanf(readpipe, "%*s %c ", &scanChar) != 1) return(x);
		breakpoints[x].enabled = (scanChar == 'y');
		if(fscanf(readpipe, "%x", & breakpoints[x].segment) != 1) return(x);
		if(fscanf(readpipe, ":%x", &breakpoints[x].offset) < 1){
			breakpoints[x].offset  = breakpoints[x].segment;
			breakpoints[x].segment = 0;
		}
	}
	return(x);
}


int string_read( char *buffer, char *string, const char delimiter ){
	register char *current, *end, c;
	register int n;

	n = 0;
	current = string;
	end = string + STRING_LENGTH;

	while(current < end){
		c = buffer[n]; n++;
		if((c == delimiter) || (c == EOF)){
			do{
				--current;
			} while((current >= string) && (isspace(*current) != 0));
			*(++current) = '\0';
			return(0);
		}
		*current = c;
		++current;
	}
	*(--end) = '\0';
	return(0);
}


int remove_word( char *buffer, int current, int max ){
	for(; current < max
		&& buffer[current] != '\0'
		&& buffer[current] != '\n'
		&& buffer[current] != '\t'
		&& buffer[current] != ' '; current++);
	while(buffer[current] == ' ' || buffer[current] == '\t') ++current;
	return(current);
}


int instruction_read( ){
	char buffer[200];
	int n;

	if(bochs_offline) return(1);

	n = 1;
#ifdef __OLD_BOCHS__
	fgets(buffer, 200, readpipe);
#endif
	fgets(buffer, 200, readpipe);
	if(sscanf(&buffer[n], "%*s %x:%x: ", &i_virtual_segment, &i_virtual_offset) != 2) return(1);
	n = remove_word(buffer, n, 200); n = remove_word(buffer, n, 200);

	if(buffer[n] == '('){
		n = remove_word(buffer, n, 200); n = remove_word(buffer, n, 200);
	}
	sscanf(&buffer[n], "%*x%n: ", &i_size);
	i_size/=2;

	n = remove_word(buffer, n, 200);
	string_read(&buffer[n], i_description, '\n');
	if(i_description == NULL) return(1);
	return(0);
}


int prompt_read( ){
	char c;

	if(bochs_offline) return(1);
#ifdef __DEBUG__
	printf("DEBUG: reading prompt... ");
#endif
	for(c=fgetc(readpipe); c != '<' && c != EOF; c=fgetc(readpipe));
	if(c != '<') return(1);
	if(fscanf(readpipe, "bochs:%u>", &p_step) != 1) return(1);
#ifdef __DEBUG__
	printf("OK\n");
#endif
	return(0);
}


int cpu_read ( ){

	if(bochs_offline) return(0);

    return(fscanf(readpipe, 
		" eax:%x "
	    	"ebx:%x "
		"ecx:%x "
		"edx:%x "
		"ebp:%x "
		"esi:%x "
		"edi:%x "
		"esp:%x "
		"eflags:%x "
		"eip:%x "
		"cs:s=%x, dl=%x, dh=%x, valid=%i "
		"ss:s=%x, dl=%x, dh=%x, valid=%i "
                "ds:s=%x, dl=%x, dh=%x, valid=%i "
                "es:s=%x, dl=%x, dh=%x, valid=%i "
                "fs:s=%x, dl=%x, dh=%x, valid=%i "
                "gs:s=%x, dl=%x, dh=%x, valid=%i "
                "ldtr:s=%x, dl=%x, dh=%x, valid=%i "
                "tr:s=%x, dl=%x, dh=%x, valid=%i "
		"gdtr:base=%x, limit=%x "
		"idtr:base=%x, limit=%x "
		"dr0:%x "
		"dr1:%x "
                "dr2:%x "
                "dr3:%x "
//                "dr4:%x "
//                "dr5:%x "
                "dr6:%x "
                "dr7:%x "
                "tr3:%x "
                "tr4:%x "
                "tr5:%x "
                "tr6:%x "
                "tr7:%x "
                "cr0:%x "
                "cr1:%x "
                "cr2:%x "
                "cr3:%x "
                "cr4:%x "
		"inhibit_int:%i "
		"done",
		&cpu.eax,
		&cpu.ebx,
		&cpu.ecx,
		&cpu.edx,
		&cpu.ebp,
		&cpu.esi,
		&cpu.edi,
		&cpu.esp,
		&cpu.eflags,
		&cpu.eip,
		&cpu.cs.segment,		
		&cpu.cs.descriptor[0],
		&cpu.cs.descriptor[1],
		&cpu.cs.valid,
                &cpu.ss.segment,
                &cpu.ss.descriptor[0],
                &cpu.ss.descriptor[1],
                &cpu.ss.valid,
                &cpu.ds.segment,
                &cpu.ds.descriptor[0],
                &cpu.ds.descriptor[1],
                &cpu.ds.valid,
                &cpu.es.segment,
                &cpu.es.descriptor[0],
                &cpu.es.descriptor[1],
                &cpu.es.valid,
                &cpu.fs.segment,
                &cpu.fs.descriptor[0],
                &cpu.fs.descriptor[1],
                &cpu.fs.valid,
                &cpu.gs.segment,
                &cpu.gs.descriptor[0],
                &cpu.gs.descriptor[1],
                &cpu.gs.valid,
                &cpu.ldtr.segment,
                &cpu.ldtr.descriptor[0],
                &cpu.ldtr.descriptor[1],
                &cpu.ldtr.valid,
                &cpu.tr.segment,
                &cpu.tr.descriptor[0],
                &cpu.tr.descriptor[1],
                &cpu.tr.valid,
		&cpu.gdtr.base,
		&cpu.gdtr.limit,
		&cpu.idtr.base,
		&cpu.idtr.limit,
		&cpu.dr0,
		&cpu.dr1,
		&cpu.dr2,
		&cpu.dr3,
//		&cpu.dr4,
//		&cpu.dr5,
		&cpu.dr6,
		&cpu.dr7,
		&cpu.tr3,
		&cpu.tr4,
		&cpu.tr5,
		&cpu.tr6,
		&cpu.tr7,
		&cpu.cr0,
		&cpu.cr1,
		&cpu.cr2,
		&cpu.cr3,
		&cpu.cr4,
		&cpu.inhibit)  == (sizeof(struct cpu_t) / sizeof(int)) );
}
